@@ -2,8 +2,6 @@
 
 Copyright (c) 1999-2004 Ng Pheng Siong. All rights reserved."""
 
-RCS_id='$Id: RSA.py 299 2005-06-09 17:32:28Z heikki $'
-
 import sys
 import util, BIO, Err, m2
 
@@ -21,17 +19,17 @@ class RSA:
     """
     RSA Key Pair.
     """
+
+    m2_rsa_free = m2.rsa_free
+
     def __init__(self, rsa, _pyfree=0):
         assert m2.rsa_type_check(rsa), "'rsa' type error"
         self.rsa = rsa
         self._pyfree = _pyfree
-
+        
     def __del__(self):
-        try:
-            if self._pyfree:
-                m2.rsa_free(self.rsa)
-        except AttributeError:
-            pass
+        if getattr(self, '_pyfree', 0):
+            self.m2_rsa_free(self.rsa)
 
     def __len__(self):
         return m2.rsa_size(self.rsa) << 3
@@ -134,8 +132,8 @@ class RSA:
         """
         Save the key pair to a file in DER format.
 
-        @type bio: M2Crypto.BIO.BIO
-        @param bio: M2Crypto.BIO.BIO object to save key to.
+        @type file: str
+        @param file: Filename to save key to
         """
         bio = BIO.openfile(file, 'wb')
         return self.save_key_der_bio(bio)
@@ -162,6 +160,50 @@ class RSA:
     def check_key(self):
         return m2.rsa_check_key(self.rsa)
 
+    def sign(self, digest, algo='sha1'):
+        """
+        Signs a digest with the private key
+
+        @type digest: str
+        @param digest: A digest created by using the digest method
+
+        @type algo: str
+        @param algo: The method that created the digest.
+        Legal values are 'sha1','sha224', 'sha256', 'ripemd160', 
+        and 'md5'.
+        
+        @return: a string which is the signature
+        """
+        digest_type = getattr(m2, 'NID_' + algo, None) 
+        if digest_type is None:
+            raise ValueError, ('unknown algorithm', algo)
+        
+        return m2.rsa_sign(self.rsa, digest, digest_type) 
+    
+    def verify(self, data, signature, algo='sha1'):
+        """
+        Verifies the signature with the public key
+
+        @type data: str
+        @param data: Data that has been signed
+
+        @type signature: str
+        @param signature: The signature signed with the private key
+
+        @type algo: str
+        @param algo: The method use to create digest from the data 
+        before it was signed.  Legal values are 'sha1','sha224',
+        'sha256', 'ripemd160', and 'md5'.
+
+        @return: True or False, depending on whether the signature was
+        verified.  
+        """
+        digest_type = getattr(m2, 'NID_' + algo, None)
+        if digest_type is None:
+            raise ValueError, ('unknown algorithm', algo)
+        
+        return m2.rsa_verify(self.rsa, data, signature, digest_type) 
+
 
 class RSA_pub(RSA):
 
