@@ -2,8 +2,6 @@
 
 Copyright (c) 1999-2003 Ng Pheng Siong. All rights reserved."""
 
-RCS_id='$Id: Rand.py 299 2005-06-09 17:32:28Z heikki $'
-
 import m2
 
 rand_seed           = m2.rand_seed
@@ -13,7 +11,4 @@ save_file           = m2.rand_save_file
 rand_bytes          = m2.rand_bytes
 rand_pseudo_bytes   = m2.rand_pseudo_bytes
 
-def estimate_entropy(buf):
-    raise NotImplementedError
-
 
