@@ -1,11 +1,11 @@
 """
-M2Crypto.SSL.Checker
+SSL peer certificate checking routines
 
-Copyright (c) 2004-2005 Open Source Applications Foundation.
+Copyright (c) 2004-2007 Open Source Applications Foundation.
 All rights reserved.
 """
 
-from M2Crypto import util, EVP
+from M2Crypto import util, EVP, m2
 import re
 
 class SSLVerificationError(Exception):
@@ -48,11 +48,13 @@ class WrongHost(SSLVerificationError):
 
 
 class Checker:
+    
+    numericIpMatch = re.compile('^[0-9]+(\.[0-9]+)*$')
+    
     def __init__(self, host=None, peerCertHash=None, peerCertDigest='sha1'):
         self.host = host
         self.fingerprint = peerCertHash
         self.digest = peerCertDigest
-        self.numericIpMatch = re.compile('^[0-9]+(\.[0-9]+)*$')
 
     def __call__(self, peerCert, host=None):
         if peerCert is None:
@@ -78,12 +80,12 @@ class Checker:
 
         if self.host:
             hostValidationPassed = False
+            self.useSubjectAltNameOnly = False
 
-            # XXX subjectAltName might contain multiple fields
-            # subjectAltName=DNS:somehost
+            # subjectAltName=DNS:somehost[, ...]*
             try:
                 subjectAltName = peerCert.get_ext('subjectAltName').get_value()
-                if not self._match(self.host, subjectAltName, True):
+                if not self._splitSubjectAltName(self.host, subjectAltName):
                     raise WrongHost(expectedHost=self.host, 
                                     actualHost=subjectAltName,
                                     fieldName='subjectAltName')
@@ -91,30 +93,74 @@ class Checker:
             except LookupError:
                 pass
 
-            # commonName=somehost
-            if not hostValidationPassed:
-                try:
-                    commonName = peerCert.get_subject().CN
-                    if not self._match(self.host, commonName):
-                        raise WrongHost(expectedHost=self.host,
-                                        actualHost=commonName,
-                                        fieldName='commonName')
-                except AttributeError:
+            # commonName=somehost[, ...]*
+            if not self.useSubjectAltNameOnly and not hostValidationPassed:
+                hasCommonName = False
+                commonNames = ''
+                for entry in peerCert.get_subject().get_entries_by_nid(m2.NID_commonName):
+                    hasCommonName = True
+                    commonName = entry.get_data().as_text()
+                    if not commonNames:
+                        commonNames = commonName
+                    else:
+                        commonNames += ',' + commonName
+                    if self._match(self.host, commonName):
+                        hostValidationPassed = True
+                        break
+
+                if not hasCommonName:
                     raise WrongCertificate('no commonName in peer certificate')
 
+                if not hostValidationPassed:
+                    raise WrongHost(expectedHost=self.host,
+                                    actualHost=commonNames,
+                                    fieldName='commonName')
+
         return True
 
-    def _match(self, host, certHost, subjectAltName=False):
+    def _splitSubjectAltName(self, host, subjectAltName):
         """
         >>> check = Checker()
-        >>> check._match(host='my.example.com', certHost='DNS:my.example.com', subjectAltName=True)
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:my.example.com')
         True
-        >>> check._match(host='my.example.com', certHost='DNS:*.example.com', subjectAltName=True)
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:*.example.com')
         True
-        >>> check._match(host='my.example.com', certHost='DNS:m*.example.com', subjectAltName=True)
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:m*.example.com')
         True
-        >>> check._match(host='my.example.com', certHost='DNS:m*ample.com', subjectAltName=True)
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:m*ample.com')
         False
+        >>> check.useSubjectAltNameOnly
+        True
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:m*ample.com, othername:<unsupported>')
+        False
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:m*ample.com, DNS:my.example.org')
+        False
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:m*ample.com, DNS:my.example.com')
+        True
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='DNS:my.example.com, DNS:my.example.org')
+        True
+        >>> check.useSubjectAltNameOnly
+        True
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='')
+        False
+        >>> check._splitSubjectAltName(host='my.example.com', subjectAltName='othername:<unsupported>')
+        False
+        >>> check.useSubjectAltNameOnly
+        False
+        """
+        self.useSubjectAltNameOnly = False
+        for certHost in subjectAltName.split(','):
+            certHost = certHost.lower().strip()
+            if certHost[:4] == 'dns:':
+                self.useSubjectAltNameOnly = True
+                if self._match(host, certHost[4:]):
+                    return True
+        return False
+        
+
+    def _match(self, host, certHost):
+        """
+        >>> check = Checker()
         >>> check._match(host='my.example.com', certHost='my.example.com')
         True
         >>> check._match(host='my.example.com', certHost='*.example.com')
@@ -134,17 +180,12 @@ class Checker:
         >>> check._match(host='1234', certHost='1234')
         True
         """
-        # XXX See RFC 2818 and 3280 for matching rules, this is not
-        # XXX yet complete.
+        # XXX See RFC 2818 and 3280 for matching rules, this is may not
+        # XXX yet be complete.
 
         host = host.lower()
         certHost = certHost.lower()
 
-        if subjectAltName:
-            if certHost[:4] != 'dns:':
-                return False
-            certHost = certHost[4:]
-        
         if host == certHost:
             return True
 
