@@ -1,8 +1,10 @@
 /* Copyright (c) 1999 Ng Pheng Siong. All rights reserved. */
-/* $Id: _lib.h 522 2007-05-08 22:21:51Z heikki $ */
+/* $Id: _lib.h 594 2007-10-12 21:51:11Z heikki $ */
 
-#if PY_VERSION_HEX < 0x02050000
+#if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
 typedef int Py_ssize_t;
+#define PY_SSIZE_T_MAX INT_MAX
+#define PY_SSIZE_T_MIN INT_MIN
 #endif
 
 typedef struct _blob {
