@@ -27,12 +27,17 @@ def verify_cb_new_function(ok, store):
     try:
         assert not ok
         err = store.get_error()
-        assert err == m2.X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT or \
-               err == m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY or \
-               err == m2.X509_V_ERR_CERT_UNTRUSTED or \
-               err == m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE
+        assert err in [m2.X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT,
+                       m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY,
+                       m2.X509_V_ERR_CERT_UNTRUSTED,
+                       m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE]
         app_data = m2.x509_store_ctx_get_app_data(store.ctx)
         assert app_data
+        x509 = store.get_current_cert()
+        assert x509
+        stack = store.get1_chain()
+        assert len(stack) == 1
+        assert stack[0].as_pem() == x509.as_pem()
     except AssertionError, e:
         # If we let exceptions propagate from here the
         # caller may see strange errors. This is cleaner.
@@ -43,8 +48,9 @@ class VerifyCB:
     def __call__(self, ok, store):
         return verify_cb_new_function(ok, store)
 
+sleepTime = float(os.getenv('M2CRYPTO_TEST_SSL_SLEEP', 0.5))
 
-class SSLClientTestCase(unittest.TestCase):
+class BaseSSLClientTestCase(unittest.TestCase):
 
     def start_server(self, args):
         pid = os.fork()
@@ -58,7 +64,7 @@ class SSLClientTestCase(unittest.TestCase):
                 os.chdir('..')
                 
         else:
-            time.sleep(0.5)
+            time.sleep(sleepTime)
             return pid
 
     def stop_server(self, pid):
@@ -91,6 +97,149 @@ class SSLClientTestCase(unittest.TestCase):
         global srv_port
         srv_port = srv_port - 1
 
+
+class PassSSLClientTestCase(BaseSSLClientTestCase):
+        
+    def test_pass(self):
+        pass
+
+class HttpslibSSLClientTestCase(BaseSSLClientTestCase):
+
+    def test_HTTPSConnection(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            c = httpslib.HTTPSConnection(srv_host, srv_port)
+            c.request('GET', '/')
+            data = c.getresponse().read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPSConnection_resume_session(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.load_verify_locations(cafile='tests/ca.pem')
+            ctx.load_cert('tests/x509.pem')
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 1)
+            ctx.set_session_cache_mode(m2.SSL_SESS_CACHE_CLIENT)
+            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
+            c.request('GET', '/')
+            ses = c.get_session()
+            t = ses.as_text()
+            data = c.getresponse().read()
+            c.close()
+            
+            ctx2 = SSL.Context()
+            ctx2.load_verify_locations(cafile='tests/ca.pem')
+            ctx2.load_cert('tests/x509.pem')
+            ctx2.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 1)
+            ctx2.set_session_cache_mode(m2.SSL_SESS_CACHE_CLIENT)
+            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx2)
+            c.set_session(ses)
+            c.request('GET', '/')
+            ses2 = c.get_session()
+            t2 = ses2.as_text()
+            data = c.getresponse().read()
+            c.close()
+            assert t == t2, "Sessions did not match"
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPSConnection_secure_context(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/ca.pem')
+            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
+            c.request('GET', '/')
+            data = c.getresponse().read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPSConnection_secure_context_fail(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/server.pem')
+            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
+            self.assertRaises(SSL.SSLError, c.request, 'GET', '/')
+            c.close()
+        finally:
+            self.stop_server(pid)
+
+    def test_HTTPS(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            c = httpslib.HTTPS(srv_host, srv_port)
+            c.putrequest('GET', '/')
+            c.putheader('Accept', 'text/html')
+            c.putheader('Accept', 'text/plain')
+            c.endheaders()
+            err, msg, headers = c.getreply()
+            assert err == 200, err
+            f = c.getfile()
+            data = f.read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPS_secure_context(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/ca.pem')
+            c = httpslib.HTTPS(srv_host, srv_port, ssl_context=ctx)
+            c.putrequest('GET', '/')
+            c.putheader('Accept', 'text/html')
+            c.putheader('Accept', 'text/plain')
+            c.endheaders()
+            err, msg, headers = c.getreply()
+            assert err == 200, err
+            f = c.getfile()
+            data = f.read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPS_secure_context_fail(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('tests/server.pem')
+            c = httpslib.HTTPS(srv_host, srv_port, ssl_context=ctx)
+            c.putrequest('GET', '/')
+            c.putheader('Accept', 'text/html')
+            c.putheader('Accept', 'text/plain')
+            self.assertRaises(SSL.SSLError, c.endheaders)
+            c.close()
+        finally:
+            self.stop_server(pid)
+
+
+class MiscSSLClientTestCase(BaseSSLClientTestCase):
+
+    def test_no_connection(self):
+        ctx = SSL.Context()
+        s = SSL.Connection(ctx)
+        
     def test_server_simple(self):
         pid = self.start_server(self.args)
         try:
@@ -130,6 +279,36 @@ class SSLClientTestCase(unittest.TestCase):
         finally:
             self.stop_server(pid)
 
+    def test_server_simple_timeouts(self):
+        pid = self.start_server(self.args)
+        try:
+            self.assertRaises(ValueError, SSL.Context, 'tlsv5')
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            
+            r = s.get_socket_read_timeout()
+            w = s.get_socket_write_timeout()
+            assert r.sec == 0, r.sec
+            assert r.microsec == 0, r.microsec
+            assert w.sec == 0, w.sec
+            assert w.microsec == 0, w.microsec
+
+            s.set_socket_read_timeout(SSL.timeout())
+            s.set_socket_write_timeout(SSL.timeout(909,9))
+            r = s.get_socket_read_timeout()
+            w = s.get_socket_write_timeout()
+            assert r.sec == 600, r.sec
+            assert r.microsec == 0, r.microsec
+            assert w.sec == 909, w.sec
+            #assert w.microsec == 9, w.microsec XXX 4000
+            
+            s.connect(self.srv_addr)
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
     def test_tls1_nok(self):
         self.args.append('-no_tls1')
         pid = self.start_server(self.args)
@@ -259,6 +438,8 @@ class SSLClientTestCase(unittest.TestCase):
             s.connect(self.srv_addr)
             data = self.http_get(s)
             
+            assert s.get_cipher().name() == 'EXP-RC4-MD5', s.get_cipher().name()
+            
             cipher_stack = s.get_ciphers()
             assert cipher_stack[0].name() == 'EXP-RC4-MD5', cipher_stack[0].name()
             self.assertRaises(IndexError, cipher_stack.__getitem__, 2)
@@ -546,167 +727,6 @@ class SSLClientTestCase(unittest.TestCase):
         finally:
             self.stop_server(pid)
 
-    def test_HTTPSConnection(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            c = httpslib.HTTPSConnection(srv_host, srv_port)
-            c.request('GET', '/')
-            data = c.getresponse().read()
-            c.close()
-        finally:
-            self.stop_server(pid)
-        self.failIf(string.find(data, 's_server -quiet -www') == -1)
-
-    def test_HTTPSConnection_secure_context(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            ctx = SSL.Context()
-            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('tests/ca.pem')
-            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
-            c.request('GET', '/')
-            data = c.getresponse().read()
-            c.close()
-        finally:
-            self.stop_server(pid)
-        self.failIf(string.find(data, 's_server -quiet -www') == -1)
-
-    def test_HTTPSConnection_secure_context_fail(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            ctx = SSL.Context()
-            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('tests/server.pem')
-            c = httpslib.HTTPSConnection(srv_host, srv_port, ssl_context=ctx)
-            self.assertRaises(SSL.SSLError, c.request, 'GET', '/')
-            c.close()
-        finally:
-            self.stop_server(pid)
-
-    def test_HTTPS(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            c = httpslib.HTTPS(srv_host, srv_port)
-            c.putrequest('GET', '/')
-            c.putheader('Accept', 'text/html')
-            c.putheader('Accept', 'text/plain')
-            c.endheaders()
-            err, msg, headers = c.getreply()
-            assert err == 200, err
-            f = c.getfile()
-            data = f.read()
-            c.close()
-        finally:
-            self.stop_server(pid)
-        self.failIf(string.find(data, 's_server -quiet -www') == -1)
-
-    def test_HTTPS_secure_context(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            ctx = SSL.Context()
-            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('tests/ca.pem')
-            c = httpslib.HTTPS(srv_host, srv_port, ssl_context=ctx)
-            c.putrequest('GET', '/')
-            c.putheader('Accept', 'text/html')
-            c.putheader('Accept', 'text/plain')
-            c.endheaders()
-            err, msg, headers = c.getreply()
-            assert err == 200, err
-            f = c.getfile()
-            data = f.read()
-            c.close()
-        finally:
-            self.stop_server(pid)
-        self.failIf(string.find(data, 's_server -quiet -www') == -1)
-
-    def test_HTTPS_secure_context_fail(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import httpslib
-            ctx = SSL.Context()
-            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-            ctx.load_verify_locations('tests/server.pem')
-            c = httpslib.HTTPS(srv_host, srv_port, ssl_context=ctx)
-            c.putrequest('GET', '/')
-            c.putheader('Accept', 'text/html')
-            c.putheader('Accept', 'text/plain')
-            self.assertRaises(SSL.SSLError, c.endheaders)
-            c.close()
-        finally:
-            self.stop_server(pid)
-
-    def test_urllib(self):
-        pid = self.start_server(self.args)
-        try:
-            from M2Crypto import m2urllib
-            url = m2urllib.FancyURLopener()
-            url.addheader('Connection', 'close')
-            u = url.open('https://%s:%s/' % (srv_host, srv_port))
-            data = u.read()
-            u.close()
-        finally:
-            self.stop_server(pid)
-        self.failIf(string.find(data, 's_server -quiet -www') == -1)
-
-    # XXX Don't actually know how to use m2urllib safely!
-    #def test_urllib_secure_context(self):
-    #def test_urllib_secure_context_fail(self):
-
-    # XXX Don't actually know how to use m2urllib safely!
-    #def test_urllib_safe_context(self):
-    #def test_urllib_safe_context_fail(self):
-
-    if sys.version_info >= (2,4):
-        def test_urllib2(self):
-            pid = self.start_server(self.args)
-            try:
-                from M2Crypto import m2urllib2
-                opener = m2urllib2.build_opener()
-                opener.addheaders = [('Connection', 'close')]
-                u = opener.open('https://%s:%s/' % (srv_host, srv_port))
-                data = u.read()
-                u.close()
-            finally:
-                self.stop_server(pid)
-            self.failIf(string.find(data, 's_server -quiet -www') == -1)
-    
-        def test_urllib2_secure_context(self):
-            pid = self.start_server(self.args)
-            try:
-                ctx = SSL.Context()
-                ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-                ctx.load_verify_locations('tests/ca.pem')
-                
-                from M2Crypto import m2urllib2
-                opener = m2urllib2.build_opener(ctx)
-                opener.addheaders = [('Connection', 'close')]           
-                u = opener.open('https://%s:%s/' % (srv_host, srv_port))
-                data = u.read()
-                u.close()
-            finally:
-                self.stop_server(pid)
-            self.failIf(string.find(data, 's_server -quiet -www') == -1)
-    
-        def test_urllib2_secure_context_fail(self):
-            pid = self.start_server(self.args)
-            try:
-                ctx = SSL.Context()
-                ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
-                ctx.load_verify_locations('tests/server.pem')
-                
-                from M2Crypto import m2urllib2
-                opener = m2urllib2.build_opener(ctx)
-                opener.addheaders = [('Connection', 'close')]
-                self.assertRaises(SSL.SSLError, opener.open, 'https://%s:%s/' % (srv_host, srv_port))
-            finally:
-                self.stop_server(pid)
-
     def test_blocking0(self):
         pid = self.start_server(self.args)
         try:
@@ -755,20 +775,6 @@ class SSLClientTestCase(unittest.TestCase):
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
     def test_makefile_err(self):
-        def http_get(s):
-            s.send('GET / HTTP/1.0\n\n') 
-            resp = ''
-            while 1:
-                try:
-                    r = s.recv(4096)
-                    if not r:
-                        break
-                except SSL.SSLError, e: # s_server throws an 'unexpected eof'...
-                    #print e
-                    break
-                resp = resp + r 
-            return resp
-
         pid = self.start_server(self.args)
         try:
             ctx = SSL.Context()
@@ -778,12 +784,10 @@ class SSLClientTestCase(unittest.TestCase):
             except SSL.SSLError, e:
                 assert 0, e
             f = s.makefile()
-            data = http_get(s)
+            data = self.http_get(s)
             s.close()
             del f
-            f = None
             del s
-            s = None
             err_code = Err.peek_error_code()
             assert not err_code, 'Unexpected error: %s' % err_code
             err = Err.get_error()
@@ -792,6 +796,97 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+
+class UrllibSSLClientTestCase(BaseSSLClientTestCase):
+
+    def test_urllib(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import m2urllib
+            url = m2urllib.FancyURLopener()
+            url.addheader('Connection', 'close')
+            u = url.open('https://%s:%s/' % (srv_host, srv_port))
+            data = u.read()
+            u.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    # XXX Don't actually know how to use m2urllib safely!
+    #def test_urllib_secure_context(self):
+    #def test_urllib_secure_context_fail(self):
+
+    # XXX Don't actually know how to use m2urllib safely!
+    #def test_urllib_safe_context(self):
+    #def test_urllib_safe_context_fail(self):
+
+
+class Urllib2SSLClientTestCase(BaseSSLClientTestCase):
+
+    if sys.version_info >= (2,4):
+        def test_urllib2(self):
+            pid = self.start_server(self.args)
+            try:
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener()
+                opener.addheaders = [('Connection', 'close')]
+                u = opener.open('https://%s:%s/' % (srv_host, srv_port))
+                data = u.read()
+                u.close()
+            finally:
+                self.stop_server(pid)
+            self.failIf(string.find(data, 's_server -quiet -www') == -1)
+    
+        def test_urllib2_secure_context(self):
+            pid = self.start_server(self.args)
+            try:
+                ctx = SSL.Context()
+                ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+                ctx.load_verify_locations('tests/ca.pem')
+                
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener(ctx)
+                opener.addheaders = [('Connection', 'close')]           
+                u = opener.open('https://%s:%s/' % (srv_host, srv_port))
+                data = u.read()
+                u.close()
+            finally:
+                self.stop_server(pid)
+            self.failIf(string.find(data, 's_server -quiet -www') == -1)
+    
+        def test_urllib2_secure_context_fail(self):
+            pid = self.start_server(self.args)
+            try:
+                ctx = SSL.Context()
+                ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+                ctx.load_verify_locations('tests/server.pem')
+                
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener(ctx)
+                opener.addheaders = [('Connection', 'close')]
+                self.assertRaises(SSL.SSLError, opener.open, 'https://%s:%s/' % (srv_host, srv_port))
+            finally:
+                self.stop_server(pid)
+
+        def test_z_urllib2_opener(self):
+            pid = self.start_server(self.args)
+            try:
+                ctx = SSL.Context()
+
+                from M2Crypto import m2urllib2
+                opener = m2urllib2.build_opener(ctx)
+                m2urllib2.install_opener(opener)
+                req = m2urllib2.Request('https://%s:%s/' % (srv_host, srv_port))
+                u = m2urllib2.urlopen(req)
+                data = u.read()
+                u.close()
+            finally:
+                self.stop_server(pid)
+            self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+
+class TwistedSSLClientTestCase(BaseSSLClientTestCase):
+
     def test_twisted_wrapper(self):
         # Test only when twisted and ZopeInterfaces are present
         try:
@@ -862,12 +957,21 @@ class ContextTestCase(unittest.TestCase):
     def test_ctx_load_verify_locations(self):
         ctx = SSL.Context()
         self.assertRaises(ValueError, ctx.load_verify_locations, None, None)
+        
+    def test_map(self):
+        from M2Crypto.SSL.Context import map, _ctxmap
+        assert isinstance(map(), _ctxmap)
 
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(CheckerTestCase))
     suite.addTest(unittest.makeSuite(ContextTestCase))
-    suite.addTest(unittest.makeSuite(SSLClientTestCase))
+    suite.addTest(unittest.makeSuite(PassSSLClientTestCase))
+    suite.addTest(unittest.makeSuite(HttpslibSSLClientTestCase))
+    suite.addTest(unittest.makeSuite(UrllibSSLClientTestCase))
+    suite.addTest(unittest.makeSuite(Urllib2SSLClientTestCase))
+    suite.addTest(unittest.makeSuite(MiscSSLClientTestCase))
+    suite.addTest(unittest.makeSuite(TwistedSSLClientTestCase))
     return suite    
     
 
