@@ -11,7 +11,7 @@ Author: Heikki Toivonen
 
 import unittest
 import os, time, base64, sys
-from M2Crypto import X509, EVP, RSA, Rand, ASN1, m2, util
+from M2Crypto import X509, EVP, RSA, Rand, ASN1, m2, util, BIO
 
 class X509TestCase(unittest.TestCase):
 
@@ -42,11 +42,15 @@ class X509TestCase(unittest.TestCase):
         assert x.verify(pk2)
         return x, pk
 
-    def check_ext(self):
+    def test_ext(self):
         self.assertRaises(ValueError, X509.new_extension,
                           'subjectKeyIdentifier', 'hash')
+        ext = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
+        assert ext.get_value() == 'DNS:foobar.example.com'
+        assert ext.get_value(indent=2) == '  DNS:foobar.example.com'
+        assert ext.get_value(flag=m2.X509V3_EXT_PARSE_UNKNOWN) == 'DNS:foobar.example.com'
 
-    def check_extstack(self):
+    def test_extstack(self):
         # new
         ext1 = X509.new_extension('subjectAltName', 'DNS:foobar.example.com')
         ext2 = X509.new_extension('nsComment', 'Hello there')
@@ -73,7 +77,7 @@ class X509TestCase(unittest.TestCase):
         assert len(extstack) == 2
         assert(extstack[1].get_name() == 'nsComment')
 
-    def check_x509_name(self):
+    def test_x509_name(self):
         n = X509.X509_Name()
         n.C = 'US' # It seems this actually needs to be a real 2 letter country code
         assert n.C == 'US'
@@ -104,22 +108,62 @@ class X509TestCase(unittest.TestCase):
         n.add_entry_by_txt(field="CN", type=ASN1.MBSTRING_ASC,
                            entry="Proxy", len=-1, loc=-1, set=0)
         assert len(n) == 12, len(n)
+        assert n.entry_count() == 12, n.entry_count()
         assert n.as_text() == 'C=US, ST=State or Province, L=locality name, O=orhanization name, OU=org unit, CN=common name/emailAddress=bob@example.com/serialNumber=1234, SN=surname, GN=given name, GN=name given, CN=Proxy', '"%s"' % n.as_text()
 
         self.assertRaises(AttributeError, n.__getattr__, 'foobar')
         n.foobar = 1
         assert n.foobar == 1, n.foobar
-                           
-                           
-    def check_mkreq(self):
+        
+        # X509_Name_Entry tests
+        l = 0
+        for entry in n:
+            assert isinstance(entry, X509.X509_Name_Entry), entry
+            assert isinstance(entry.get_object(), ASN1.ASN1_Object), entry
+            assert isinstance(entry.get_data(), ASN1.ASN1_String), entry
+            l += 1
+        assert l == 12, l
+        
+        l = 0
+        for cn in n.get_entries_by_nid(m2.NID_commonName):
+            assert isinstance(cn, X509.X509_Name_Entry), cn
+            assert isinstance(cn.get_object(), ASN1.ASN1_Object), cn
+            data = cn.get_data()
+            assert isinstance(data, ASN1.ASN1_String), data
+            t = data.as_text()
+            assert t == "common name" or t == "Proxy", t
+            l += 1
+        assert l == 2, l
+
+        cn.set_data("Hello There!")
+        assert cn.get_data().as_text() == "Hello There!", cn.get_data().as_text()
+
+        assert n.as_hash() == 1697185131
+
+    def test_mkreq(self):
         (req, _) = self.mkreq(512)
         req.save_pem('tests/tmp_request.pem')
         req2 = X509.load_request('tests/tmp_request.pem')
         os.remove('tests/tmp_request.pem')
+        req.save('tests/tmp_request.pem')
+        req3 = X509.load_request('tests/tmp_request.pem')
+        os.remove('tests/tmp_request.pem')
+        req.save('tests/tmp_request.der', format=X509.FORMAT_DER)
+        req4 = X509.load_request('tests/tmp_request.der',
+                format=X509.FORMAT_DER)
+        os.remove('tests/tmp_request.der')
         assert req.as_pem() == req2.as_pem()
         assert req.as_text() == req2.as_text()
+        assert req.as_der() == req2.as_der()
+        assert req.as_pem() == req3.as_pem()
+        assert req.as_text() == req3.as_text()
+        assert req.as_der() == req3.as_der()
+        assert req.as_pem() == req4.as_pem()
+        assert req.as_text() == req4.as_text()
+        assert req.as_der() == req4.as_der()
 
-    def check_mkcert(self):
+
+    def test_mkcert(self):
         req, pk = self.mkreq(512)
         pkey = req.get_pubkey()
         assert(req.verify(pkey))
@@ -195,17 +239,20 @@ class X509TestCase(unittest.TestCase):
             assert cert.check_purpose(m2.X509_PURPOSE_SSL_SERVER, 1)
             assert cert.check_purpose(m2.X509_PURPOSE_NS_SSL_SERVER, 1)
             assert cert.check_purpose(m2.X509_PURPOSE_ANY, 1)
+            assert cert.check_purpose(m2.X509_PURPOSE_SSL_SERVER, 0)
+            assert cert.check_purpose(m2.X509_PURPOSE_NS_SSL_SERVER, 0)
+            assert cert.check_purpose(m2.X509_PURPOSE_ANY, 0)
         else:
             self.assertRaises(AttributeError, cert.check_ca)
         
         return cert, pk, pkey
 
-    def check_mkcacert(self): 
+    def test_mkcacert(self): 
         cacert, pk, pkey = self.mkcacert()
         assert cacert.verify(pkey)
         
 
-    def check_mkproxycert(self): 
+    def test_mkproxycert(self): 
         cacert, pk1, pkey = self.mkcacert()
         end_entity_cert_req, pk2 = self.mkreq(512)
         end_entity_cert = self.make_eecert(cacert)
@@ -268,20 +315,88 @@ class X509TestCase(unittest.TestCase):
 
         proxycert.set_subject_name(subject_name)
         pci_ext = X509.new_extension("proxyCertInfo", 
-                                     "critical,language:Inherit all", 1, 0)
+                                     "critical,language:Inherit all", 1) # XXX leaks 8 bytes 
         proxycert.add_ext(pci_ext)
         return proxycert
     
-    def check_fingerprint(self):
+    def test_fingerprint(self):
         x509 = X509.load_cert('tests/x509.pem')
         fp = x509.get_fingerprint('sha1')
         expected = '128858B5222A5C78397530A5706233A9EB470AC4'
         assert fp == expected, '%s != %s' % (fp, expected)
 
+    def test_load_der_string(self):
+        f = open('tests/x509.der')
+        x509 = X509.load_cert_der_string(''.join(f.readlines()))
+        fp = x509.get_fingerprint('sha1')
+        expected = '128858B5222A5C78397530A5706233A9EB470AC4'
+        assert fp == expected, '%s != %s' % (fp, expected)
+
+    def test_save_der_string(self):
+        x509 = X509.load_cert('tests/x509.pem')
+        s = x509.as_der()
+        f = open('tests/x509.der')
+        s2 = f.read()
+        f.close()
+        assert s == s2, '%s != %s' % (s, s2)
+
+    def test_load(self):
+        x509 = X509.load_cert('tests/x509.pem')
+        x5092 = X509.load_cert('tests/x509.der', format=X509.FORMAT_DER)
+        assert x509.as_text() == x5092.as_text()
+        assert x509.as_pem() == x5092.as_pem()
+        assert x509.as_der() == x5092.as_der()
+        return
+    
+    def test_load_bio(self):
+        bio = BIO.openfile('tests/x509.pem')
+        bio2 = BIO.openfile('tests/x509.der')
+        x509 = X509.load_cert_bio(bio)
+        x5092 = X509.load_cert_bio(bio2, format=X509.FORMAT_DER)
+        assert x509.as_text() == x5092.as_text()
+        assert x509.as_pem() == x5092.as_pem()
+        assert x509.as_der() == x5092.as_der()
+        return
+
+    def test_load_string(self):
+        f = open('tests/x509.pem')
+        s = f.read()
+        f.close()
+        f2 = open('tests/x509.der')
+        s2 = f2.read()
+        f2.close()
+        x509 = X509.load_cert_string(s)
+        x5092 = X509.load_cert_string(s2, X509.FORMAT_DER)
+        assert x509.as_text() == x5092.as_text()
+        assert x509.as_pem() == x5092.as_pem()
+        assert x509.as_der() == x5092.as_der()
+        return
+
+    def test_save(self):
+        x509 = X509.load_cert('tests/x509.pem')
+        f = open('tests/x509.pem', 'r')
+        lTmp = f.readlines()
+        x509_pem = ''.join(lTmp[43:59])
+        f.close()
+        f = open('tests/x509.der', 'rb')
+        x509_der = f.read()
+        f.close()
+        x509.save('tests/tmpcert.pem')
+        f = open('tests/tmpcert.pem')
+        s = f.read()
+        f.close()
+        assert s == x509_pem, '%s != %s' % (s, x509_pem)
+        os.remove('tests/tmpcert.pem')
+        x509.save('tests/tmpcert.der', format=X509.FORMAT_DER)
+        f = open('tests/tmpcert.der', 'rb')
+        s = f.read()
+        f.close()
+        assert s == x509_der, '%s != %s' % (s, x509_der)
+        os.remove('tests/tmpcert.der')
 
 class X509_StackTestCase(unittest.TestCase):
     
-    def check_make_stack_from_der(self):
+    def test_make_stack_from_der(self):
         f = open("tests/der_encoded_seq.b64")
         b64 = f.read(1304)
         seq = base64.decodestring(b64)
@@ -290,8 +405,8 @@ class X509_StackTestCase(unittest.TestCase):
         
         subject = cert.get_subject() 
         assert str(subject) == "/DC=org/DC=doegrids/OU=Services/CN=host/bosshog.lbl.gov"
-    
-    def check_make_stack_check_num(self):
+
+    def test_make_stack_check_num(self):
         f = open("tests/der_encoded_seq.b64")
         b64 = f.read(1304)
         seq = base64.decodestring(b64)
@@ -304,7 +419,7 @@ class X509_StackTestCase(unittest.TestCase):
         subject = cert.get_subject() 
         assert str(subject) == "/DC=org/DC=doegrids/OU=Services/CN=host/bosshog.lbl.gov"
 
-    def check_make_stack(self):
+    def test_make_stack(self):
         stack = X509.X509_Stack()
         cert = X509.load_cert("tests/x509.pem")
         issuer = X509.load_cert("tests/ca.pem")
@@ -327,7 +442,7 @@ class X509_StackTestCase(unittest.TestCase):
         assert str(cert_subject1) == str(cert_subject2)
         assert str(issuer_subject1) == str(issuer_subject2)
     
-    def check_as_der(self):
+    def test_as_der(self):
         stack = X509.X509_Stack()
         cert = X509.load_cert("tests/x509.pem")
         issuer = X509.load_cert("tests/ca.pem")
@@ -345,10 +460,27 @@ class X509_StackTestCase(unittest.TestCase):
         assert str(issuer_subject1) == str(issuer_subject2)
         
 
+class X509_ExtTestCase(unittest.TestCase):
+    
+    def test_ext(self):
+        # With this leaks 8 bytes:
+        name = "proxyCertInfo"
+        value = "critical,language:Inherit all"
+        # With this there are no leaks:
+        #name = "nsComment"
+        #value = "Hello"
+        critical = 1
+        
+        lhash = m2.x509v3_lhash()
+        ctx = m2.x509v3_set_conf_lhash(lhash)
+        x509_ext_ptr = m2.x509v3_ext_conf(lhash, ctx, name, value)
+        x509_ext = X509.X509_Extension(x509_ext_ptr, 1)
+
 def suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(X509TestCase, 'check'))
-    suite.addTest(unittest.makeSuite(X509_StackTestCase, 'check'))
+    suite.addTest(unittest.makeSuite(X509TestCase))
+    suite.addTest(unittest.makeSuite(X509_StackTestCase))
+    #suite.addTest(unittest.makeSuite(X509_ExtTestCase))
     return suite
 
 
